#include <sourcemod>
#include <multicolors>
#include <autoexecconfig>
#include <basecomm>

public Plugin myinfo = 
{
	name = "Staff and VIP Chat", 
	author = "LanteJoula", 
	description = "Staff and VIP Chat", 
	version = "1.1", 
	url = "https://steamcommunity.com/id/lantejoula/"
};

#define DEBUG

#pragma semicolon 1

#pragma newdecls required

//ConVars

char g_ChatPrefix[256];
ConVar gConVar_Chat_Prefix;

ConVar gConVar_Witch_Chat;
ConVar gConVar_Prefix;
ConVar gConVar_Gagged;
ConVar gConVar_CommandHider;

public void OnPluginStart()
{
	//Translations
	LoadTranslations("staffvipchat.phrases");
	
	//ConVars
	AutoExecConfig_SetFile("plugin.staffvipchat");
	
	gConVar_Chat_Prefix = AutoExecConfig_CreateConVar("sm_staffvipchat_chat_prefix", "[{green}TAG{default}]", "Chat Prefix");
	
	gConVar_Witch_Chat = AutoExecConfig_CreateConVar("sm_staffvipchat_which_chat", "0", "Which of the Chats do you Want to Exist on your Server? ( 0 - BOTH CHATS | 1 - ONLY STAFF CHAT  | 2 - ONLY VIP CHAT )", 0, true, 0.0, true, 2.0);
	gConVar_Prefix = AutoExecConfig_CreateConVar("sm_staffvipchat_prefix", "1", "Enable/Disable Prefix on STAFF CHAT and VIP CHAT ( 1 - Enable | 0 - Disable )", 0, true, 0.0, true, 1.0);
	gConVar_Gagged = AutoExecConfig_CreateConVar("sm_staffvipchat_gagged", "1", "Enable/Disable Players Gagged Can Speak on STAFF CHAT and VIP CHAT ( 1 - Enable | 0 - Disable )", 0, true, 0.0, true, 1.0);
	gConVar_CommandHider = AutoExecConfig_CreateConVar("sm_staffvipchat_command_hider", "1", "Enable/Disable the Command Hider ( Removes all Commands used with ! by the Staff and VIPS ) (1 - Enable | 0 - Disable)", 0, true, 0.0, true, 1.0);
	
	AutoExecConfig_ExecuteFile();
	AutoExecConfig_CleanFile();
	
	gConVar_Chat_Prefix.AddChangeHook(OnPrefixChange);
	
	//Commands
	RegAdminCmd("sm_staffchat", Command_StaffChat, ADMFLAG_GENERIC);
	RegAdminCmd("sm_sc", Command_StaffChat, ADMFLAG_GENERIC);
	
	RegAdminCmd("sm_vipchat", Command_VIPChat, ADMFLAG_CUSTOM1);
	RegAdminCmd("sm_vc", Command_VIPChat, ADMFLAG_CUSTOM1);
	
	//Command Hider
	AddCommandListener(HideCommands, "say");
	AddCommandListener(HideCommands, "say_team");
}

//Prefix

public void SavePrefix()
{
	GetConVarString(gConVar_Chat_Prefix, g_ChatPrefix, sizeof(g_ChatPrefix));
}

public void OnConfigsExecuted()
{
	SavePrefix();
}

public void OnPrefixChange(ConVar cvar, char[] oldvalue, char[] newvalue)
{
	SavePrefix();
}

//Plugin

public Action OnClientSayCommand(int client, const char[] command, const char[] args)
{
	if (IsValidClient(client))
	{
		if (gConVar_Gagged.BoolValue)
		{
			if (BaseComm_IsClientGagged(client))
			{
				return Plugin_Continue;
			}
		}
		else
		{
			if (BaseComm_IsClientGagged(client))
			{
				CPrintToChat(client, "%s %t", g_ChatPrefix, "GAGGED");
				return Plugin_Stop;
			}
		}
		
		if (strcmp(command, "say") != 0 && strcmp(command, "say_team") != 0)
		{
			return Plugin_Continue;
		}
		
		if (gConVar_Witch_Chat.IntValue == 0) //BOTH CHATS
		{
			if (CheckCommandAccess(client, "sm_override_staff", ADMFLAG_GENERIC))
			{
				char name[MAX_NAME_LENGTH];
				GetClientName(client, name, sizeof(name));
				
				if (strcmp(command, "say_team") == 0 && args[0] == '.')
				{
					for (int i = 1; i <= MaxClients; i++)
					{
						if (IsValidClient(i))
						{
							if (CheckCommandAccess(i, "sm_override_staff", ADMFLAG_GENERIC))
							{
								if (gConVar_Prefix.BoolValue)
								{
									CPrintToChat(i, "%s %t", g_ChatPrefix, "STAFF CHAT", name, args[1]);
									return Plugin_Stop;
								}
								else
								{
									CPrintToChat(i, "%t", "STAFF CHAT", name, args[1]);
									return Plugin_Stop;
								}
							}
						}
					}
				}
			}
			
			if (CheckCommandAccess(client, "sm_override_vip", ADMFLAG_CUSTOM1))
			{
				char name[MAX_NAME_LENGTH];
				GetClientName(client, name, sizeof(name));
				
				if (strcmp(command, "say_team") == 0 && args[0] == ',')
				{
					for (int i = 1; i <= MaxClients; i++)
					{
						if (IsValidClient(i))
						{
							if (CheckCommandAccess(i, "sm_override_vip", ADMFLAG_CUSTOM1))
							{
								if (gConVar_Prefix.BoolValue)
								{
									CPrintToChat(i, "%s %t", g_ChatPrefix, "VIP CHAT", name, args[1]);
									return Plugin_Stop;
								}
								else
								{
									CPrintToChat(i, "%t", "VIP CHAT", name, args[1]);
									return Plugin_Stop;
								}
							}
						}
					}
				}
			}
			
			if (gConVar_Witch_Chat.IntValue == 1) //ONLY STAFF CHAT
			{
				if (CheckCommandAccess(client, "sm_override_staff", ADMFLAG_GENERIC))
				{
					char name[MAX_NAME_LENGTH];
					GetClientName(client, name, sizeof(name));
					
					if (strcmp(command, "say_team") == 0 && args[0] == '.')
					{
						for (int i = 1; i <= MaxClients; i++)
						{
							if (IsValidClient(i))
							{
								if (CheckCommandAccess(i, "sm_override_staff", ADMFLAG_GENERIC))
								{
									if (gConVar_Prefix.BoolValue)
									{
										CPrintToChat(i, "%s %t", g_ChatPrefix, "STAFF CHAT", name, args[1]);
										return Plugin_Stop;
									}
									else
									{
										CPrintToChat(i, "%t", "STAFF CHAT", name, args[1]);
										return Plugin_Stop;
									}
								}
							}
						}
					}
				}
			}
			
			if (gConVar_Witch_Chat.IntValue == 2) //ONLY VIP CHAT
			{
				if (CheckCommandAccess(client, "sm_override_vip", ADMFLAG_CUSTOM1))
				{
					char name[MAX_NAME_LENGTH];
					GetClientName(client, name, sizeof(name));
					
					if (strcmp(command, "say_team") == 0 && args[0] == ',')
					{
						for (int i = 1; i <= MaxClients; i++)
						{
							if (IsValidClient(i))
							{
								if (CheckCommandAccess(i, "sm_override_vip", ADMFLAG_CUSTOM1))
								{
									if (gConVar_Prefix.BoolValue)
									{
										CPrintToChat(i, "%s %t", g_ChatPrefix, "VIP CHAT", name, args[1]);
										return Plugin_Stop;
									}
									else
									{
										CPrintToChat(i, "%t", "VIP CHAT", name, args[1]);
										return Plugin_Stop;
									}
								}
							}
						}
					}
				}
			}
		}
	}
	return Plugin_Continue;
}

//Command Staff Chat

public Action Command_StaffChat(int client, int args)
{
	if (IsValidClient(client))
	{
		if (gConVar_Gagged.BoolValue)
		{
			if (BaseComm_IsClientGagged(client))
			{
				return Plugin_Continue;
			}
		}
		else
		{
			if (BaseComm_IsClientGagged(client))
			{
				CPrintToChat(client, "%s %t", g_ChatPrefix, "GAGGED");
				return Plugin_Stop;
			}
		}
		
		if (CheckCommandAccess(client, "sm_override_staff", ADMFLAG_GENERIC))
		{
			char name[MAX_NAME_LENGTH];
			GetClientName(client, name, sizeof(name));
			
			char message[256];
			GetCmdArgString(message, sizeof(message));
			
			for (int i = 1; i <= MaxClients; i++)
			{
				if (IsValidClient(i))
				{
					if (CheckCommandAccess(i, "sm_override_staff", ADMFLAG_GENERIC))
					{
						if (gConVar_Prefix.BoolValue)
							CPrintToChat(i, "%s %t", g_ChatPrefix, "STAFF CHAT", name, message);
						else
							CPrintToChat(i, "%t", "STAFF CHAT", name, message);
					}
				}
			}
		}
	}
	return Plugin_Continue;
}

//Command VIP Chat

public Action Command_VIPChat(int client, int args)
{
	if (IsValidClient(client))
	{
		if (gConVar_Gagged.BoolValue)
		{
			if (BaseComm_IsClientGagged(client))
			{
				return Plugin_Continue;
			}
		}
		else
		{
			if (BaseComm_IsClientGagged(client))
			{
				CPrintToChat(client, "%s %t", g_ChatPrefix, "GAGGED");
				return Plugin_Stop;
			}
		}
		
		if (CheckCommandAccess(client, "sm_override_vip", ADMFLAG_CUSTOM1))
		{
			char name[MAX_NAME_LENGTH];
			GetClientName(client, name, sizeof(name));
			
			char message[256];
			GetCmdArgString(message, sizeof(message));
			
			for (int i = 1; i <= MaxClients; i++)
			{
				if (IsValidClient(i))
				{
					if (CheckCommandAccess(client, "sm_override_vip", ADMFLAG_CUSTOM1))
					{
						if (gConVar_Prefix.BoolValue)
							CPrintToChat(i, "%s %t", g_ChatPrefix, "VIP CHAT", name, message);
						else
							CPrintToChat(i, "%t", "VIP CHAT", name, message);
					}
				}
			}
		}
	}
	return Plugin_Continue;
}

//Command Hider

public Action HideCommands(int client, const char[] command, int argc)
{
	if (gConVar_CommandHider.BoolValue)
	{
		if (CheckCommandAccess(client, "sm_override_staff", ADMFLAG_GENERIC) || CheckCommandAccess(client, "sm_override_vip", ADMFLAG_CUSTOM1))
		{
			if (IsChatTrigger())
				return Plugin_Handled;
		}
	}
	return Plugin_Continue;
}

//Valid Client

stock bool IsValidClient(int client)
{
	return (1 <= client <= MaxClients && IsClientInGame(client) && !IsClientSourceTV(client) && !IsFakeClient(client));
} 